Resources:
  {{funname}}Function:
    Type: AWS::Serverless::Function
    Properties:
      FunctionName: !Sub "{{funname}}-${Stage}"
      CodeUri: custombackend
{% if events is defined %}
      Events:
        {{events}}
{% endif %}
